﻿// SeiryokuComparer.cs

using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using N14PKBrowse.Data;

namespace N14PKBrowse.Compare
{
	// 比較オブジェクト 勢力
	public class SeiryokuComparer : AbstractComparer
	{
		/// <summary>比較用コールバックデリゲート 勢力</summary>
		private delegate int SubCallback( Seiryoku x, Seiryoku y, bool isAsc );

		/// <summary>比較用コールバックデリゲート 勢力</summary>
		private SubCallback _subCallback;


		/// <summary>コールバックデリゲートを取得する</summary>
		/// <param name="propertyName">プロパティ名</param>
		/// <returns>コールバックデリゲート</returns>
		public override CompareCallback GetCallback( string propertyName )
		{
			var type = this.GetType();
			// メソッド情報取得
			var mi = type.GetMethod( _CallbackPrefix + propertyName, BindingFlags.NonPublic | BindingFlags.Instance );
			if ( mi == null )
			{
				return null;
			}
			// デリゲート作成
			this._subCallback = (SubCallback)Delegate.CreateDelegate( typeof( SubCallback ), this, mi );
			this._callback = this.CompareCallback;
			return ( this._subCallback != null ) ? this._callback : null;
		}

		/// <summary>コールバックデリゲートを設定する(ICompare を使用するカスタムソート用)</summary>
		/// <param name="propertyName">プロパティ名</param>
		/// <param name="isAsc">ソートオーダー</param>
		/// <returns>デリゲートが取得できたら true を返す</returns>
		public override bool SetCallback( string propertyName, bool isAsc )
		{
			this._isAsc = isAsc;
			this._callback = this.GetCallback( propertyName );
			return ( this._callback != null ) ? true : false;
		}

		/// <summary>ソート時に使用するコールバック</summary>
		private int CompareCallback( object x, object y, bool isAsc )
		{
			return this._subCallback( x as Seiryoku, y as Seiryoku, isAsc );
		}

		#region コールバックメソッド
		private int CmpName( Seiryoku x, Seiryoku y, bool isAsc )			// 勢力名
		{
			return ( x.SortID == y.SortID ) ? 0
				: ( x.SortID == -1 ) ? isAsc ? 1 : -1
				: ( y.SortID == -1 ) ? isAsc ? -1 : 1
				: x.SortID - y.SortID;
		}
		private int CmpPtrJuuzokuSeiryoku( Seiryoku x, Seiryoku y, bool isAsc )	// 従属勢力
		{
			var x_id = x.N14pkb.Seiryokulist.GetSortIDByAddress( x.Data.PtrJuuzokuSeiryoku );
			var y_id = y.N14pkb.Seiryokulist.GetSortIDByAddress( y.Data.PtrJuuzokuSeiryoku );
			return ( x_id == y_id ) ? 0
				: ( x_id == -1 ) ? isAsc ? 1 : -1
				: ( y_id == -1 ) ? isAsc ? -1 : 1
				: x_id - y_id;
		}
		#endregion
	}
}
