﻿// Gunryaku.cs

using System;
using System.ComponentModel;

using N14PKBrowse.HelperClass;
using N14PKLibrary.Data;

namespace N14PKBrowse.Data
{
	/// <summary>軍略ビューで使用するデータクラス</summary>
	public class Gunryaku : IBrowseData, INotifyPropertyChanged
	{
		/// <summary>未加工の軍略データ</summary>
		public GunryakuData Data { get; private set; }

		/// <summary>ツール管理</summary>
		public N14PKB N14pkb { get; private set; }


		public int ID					// ID
		{
			get { return this.Data.ID; }
		}
		public uint Address				// アドレス
		{
			get { return this.Data.Address; }
		}
		public string AddressHex		// アドレス(16進表示用)
		{
			get { return this.Data.Address.ToString( "X8" ); }
		}
		public byte KoukaTypeID			// 効果タイプID 政策の効果並びに共通しているようだ
		{
			get { return this.Data.KoukaTypeID; }
			set
			{
				if ( value == this.KoukaTypeID ) { return; }
				this.Data.KoukaTypeID = value;
				this.Write();
				this.PropertyChanged.Raise( () => this.KoukaTypeID );
			}
		}
		public sbyte KoukaValue			// 効果値？
		{
			get { return this.Data.KoukaValue; }
			set
			{
				if ( value == this.KoukaValue ) { return; }
				this.Data.KoukaValue = value;
				this.Write();
				this.PropertyChanged.Raise( () => this.KoukaValue );
			}
		}
		public string Name				// 名称(シフトJIS nullターミネイト)
		{
			get { return this.Data.Name; }
			set
			{
				if ( value == this.Name ) { return; }
				this.Data.Name = value;
				this.Write();
				this.PropertyChanged.Raise( () => this.Name );
			}
		}
		public string Help				// 名称(シフトJIS nullターミネイト)
		{
			get { return this.Data.Help; }
			set
			{
				if ( value == this.Help ) { return; }
				this.Data.Help = value;
				this.Write();
				this.PropertyChanged.Raise( () => this.Help );
			}
		}


		/// <summary>コンストラクタ 軍略</summary>
		/// <param name="n14pkb">ツール管理</param>
		/// <param name="id">データID</param>
		public Gunryaku( N14PKB n14pkb, int id )
		{
			this.N14pkb = n14pkb;
			this.Data = n14pkb.N14pk.Gunryakutable[id];
		}

		/// <summary>データセーブ用のバイト列を返す</summary>
		public byte[] GetSaveData()
		{
			var record = (byte[])this.Data.GetRecord().Clone();
			return record;
		}

		/// <summary>セーブデータを反映する</summary>
		public void SetSaveData( byte[] buff )
		{
			var record = (byte[])buff.Clone();
			// 先頭4バイトの MagicNumber はゲーム起動毎に変わるため、現在の値を使用する
			BitConverter.GetBytes( this.Data.MagicNumber ).CopyTo( record, 0x00 );

			this.Data.SetRecord( record );
			this.Write();

			// ビュー更新フラグ
			this.N14pkb.View.Flags[(int)Enums.TabKind.軍略].NeedsRefresh = true;
		}

		/// <summary>メモリ書き込みとコミットするデータID登録
		/// (コミットするまでプロセスメモリには反映しない)</summary>
		private void Write()
		{
			this.N14pkb.N14pk.Gunryakutable.Write( this.ID, this.Data.GetRecord() );
			this.N14pkb.Gunryakulist.CommitID = this.ID;
		}

		/// <summary>INotifyPropertyChanged メンバ</summary>
		public event PropertyChangedEventHandler PropertyChanged;

		/// <summary>プロパティ変更時</summary>
		public void OnPropertyChanged( object sender, PropertyChangedEventArgs e )
		{
#if DEBUG
			System.Diagnostics.Debug.WriteLine( "PropertyChanged:軍略:" + ( sender as IBrowseData ).Name + " Property:" + e.PropertyName );
#endif
		}
	}
}
