﻿// ChihouData.cs

using System;
using N14PKLibrary.HelperClass;

namespace N14PKLibrary.Data
{
	/// <summary>地方データ</summary>
	public class ChihouData : IData
	{
		/// <summary>レコード</summary>
		private byte[] _rec;


		/// <summary>アドレス</summary>
		public UInt32 Address { get; set; }

		/// <summary>ID</summary>
		public Int32 ID { get; set; }


		/// <summary>00h 4バイト マジックナンバー</summary>
		public UInt32 MagicNumber
		{
			get { return BitConverter.ToUInt32( this._rec, 0x00 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x00 ); }
		}

		/// <summary>04h 7バイト 名称(シフトJIS nullターミネイト)</summary>
		public String Name
		{
			get { return MyTextConverter.BytesToString( this._rec, 0x04, 7 ); }
			set { MyTextConverter.StringToBytes( value, 7 ).CopyTo( this._rec, 0x04 ); }
		}

		/// <summary>0Bh 21バイト 読み(シフトJIS nullターミネイト)</summary>
		public String Yomi
		{
			get { return MyTextConverter.BytesToString( this._rec, 0x0B, 21 ); }
			set { MyTextConverter.StringToBytes( value, 21 ).CopyTo( this._rec, 0x0B ); }
		}

		/// <summary>20h 4バイト 隣接地方リストポインタ(2ポインタタイプ)</summary>
		public UInt32 PtrChihouList
		{
			get { return BitConverter.ToUInt32( this._rec, 0x20 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x20 ); }
		}

		/// <summary>24h 4バイト 隣接地方データ数 6固定
		/// <para>実際に隣接している数ではなく隣接地方リストに含まれるポインタデータ数</para></summary>
		public Int32 CntChihouList
		{
			get { return BitConverter.ToInt32( this._rec, 0x24 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x24 ); }
		}

		/// <summary>28h 4バイト 0固定？</summary>
		public Int32 h28_4
		{
			get { return BitConverter.ToInt32( this._rec, 0x28 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x28 ); }
		}

		/// <summary>2Ch 4バイト 0～2？</summary>
		public Int32 h2C_4
		{
			get { return BitConverter.ToInt32( this._rec, 0x2C ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x2C ); }
		}

		/// <summary>30h 4バイト</summary>
		public Int32 h30_4
		{
			get { return BitConverter.ToInt32( this._rec, 0x30 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x30 ); }
		}

		/// <summary>34h 4バイト 0固定？</summary>
		public Int32 h34_4
		{
			get { return BitConverter.ToInt32( this._rec, 0x34 ); }
			set { BitConverter.GetBytes( value ).CopyTo( this._rec, 0x34 ); }
		}

		/// <summary>データ内にレコードをセットする</summary>
		/// <remarks>各データはレコードの参照を持つ</remarks>
		/// <param name="record">レコード</param>
		public void SetRecord( Byte[] record )
		{
			this._rec = record;
		}

		/// <summary>データ内のレコードを取得する</summary>
		/// <returns>レコード</returns>
		public Byte[] GetRecord()
		{
			return this._rec;
		}
	}
}
